import { Request, Response, NextFunction } from "express";
import { prisma } from '../db';
import { assert } from "superstruct";
import { AuthorCreationData, AuthorUpdateData } from "../validation/author";
import { responde } from "../functions";
import { BookAuthorCreationData } from "../validation/book";

export async function get_all(req: Request, res: Response) {
    const authors = await prisma.author.findMany({
      where: {
        lastname: { contains: req.query.lastname?.toString(), },
      },
      orderBy: { lastname: "asc", },
      include: {
        books : {
            select : {title: true}
        }
      },
      skip: Number(req.query.skip ?? 0),
      take: Number(req.query.take ?? 20),
    });
    res.set("X-Total-Count", String(authors.length));
    res.status(200).json(authors)
}

export async function get_one(req: Request, res: Response) {
    const author = await prisma.author.findUnique({where: {id: parseInt(req.params.author_id)}})
    if (author == null) {
        responde(res, 204, "Author doens't exist.");
    }
    res.status(200).json(author)
}

export async function create_one(req: Request, res: Response) {
    assert(req.body, AuthorCreationData);
    const author = await prisma.author.create({
        data: { firstname: req.body.firstname, lastname: req.body.lastname },
    })
    responde(res, 201, "Author succesfully added", author);
}

export async function update_one(req: Request, res: Response) {
    assert(req.body, AuthorUpdateData)

    const author = await prisma.author.update({
      where: { id: parseInt(req.params.author_id) },
      data: req.body,
    });
    responde(res, 200, "Author succesfully updated.", author);
}

export async function delete_one(req : Request, res : Response) {  
    await prisma.author.delete({where:{id : parseInt(req.params.author_id)}})
    responde(res, 200, "Author succesfully deleted.");
}

export async function get_book(req : Request, res : Response) {
    const books = await prisma.book.findMany({
      where: { authorId: parseInt(req.params.author_id) },
    });
    if (books == null) {
      responde(res, 204, "Error");
    }
    res.status(200).json(books);
}
export async function add_book(req : Request, res : Response) {
    assert(req.body, BookAuthorCreationData);
    const book = await prisma.book.create({data : {title : req.body.title, authorId : parseInt(req.params.author_id)}})
    responde(res, 200, "Book successfully added.", book)
}