import { Request, Response, NextFunction } from "express";
import { prisma } from "../db";
import { assert } from "superstruct";
import { BookCreationData, BookUpdateData } from "../validation/book";
import { responde } from '../functions';
import { TagCreationData } from "../validation/tag";

export async function get_all(req: Request, res: Response) {
  const books = await prisma.book.findMany({
    where : {
      title : {contains: req.query.title?.toString()}
    },
    orderBy: {
      title:"asc"
    },
    include: {
      author: true,
      tags: true}
  });
  res.status(200).json(books);
}

export async function get_one(req: Request, res: Response) {
  const book = await prisma.book.findUnique({
    where: { id: parseInt(req.params.book_id) },
    include : {author : true, tags : true}
  });
  if (book == null) {
    responde(res, 204, "Book doens't exist.");
  }
  res.status(200).json(book);
}

export async function update_one(req: Request, res: Response) {
  assert(req.body, BookUpdateData);

  const book = await prisma.book.update({
    where: { id: parseInt(req.params.book_id) },
    data: req.body,
  });
  responde(res, 200, "Book successfully updated.", book);
}

export async function delete_one(req: Request, res: Response) {
  await prisma.book.delete({ where: { id: parseInt(req.params.book_id) } });
  responde(res, 200, "Book succesfully deleted.");
}


export async function get_tags(req: Request, res : Response) {
  const book = await prisma.book.findUnique({
    where : {id : Number(req.params.book_id)},
    select : {
      tags : true
    }
  })
  if (!book) {
    responde(res, 404, "Book doesn't exist.")
  } else {
    res.status(200).json(book.tags);
  }
}


export async function add_tag_book(req: Request, res : Response) {
  const book = await prisma.book.update({
    where: {
      id : Number(req.params.book_id)
    },
    data : {
      tags : {
        connect : {id: Number(req.params.tag_id)}
      }
    },
    include : {
      tags : true
    }
  })
  if (!book) {
      responde(res, 404, "Book doesn't exist.")
  }

  responde(res, 201, "Tag successfully added to book.", book)
}


export async function delete_tag_book(req: Request, res : Response) {
  const book = await prisma.book.update({
    where : {id : Number(req.params.book_id)},
    data : {
      tags : {
        disconnect : [{
          id : Number(req.params.tag_id)
        }]
      }
    },
    include : {
      tags : true
    }
  })
  if (!book){
    responde(res, 404, "Book doesn't exist")
  } else {
    responde(res, 200, "Fait", book)
  }
}