import { Request, Response, NextFunction } from "express";
import { prisma } from "../db";
import { assert } from "superstruct";
import { TagCreationData, TagUpdateData } from "../validation/tag";
import { responde } from "../functions";

export async function get_all(req: Request, res: Response) {
  const tags = await prisma.tag.findMany();
  res.status(200).json(tags);
}

export async function get_one(req: Request, res: Response) {
  const tag = await prisma.tag.findUnique({
    where: { id: parseInt(req.params.tag_id) },
  });
  if (tag == null) {
    responde(res, 204, "tag doens't exist.");
  }
  res.status(200).json(tag);
}

export async function create_one(req: Request, res: Response) {
  assert(req.body, TagCreationData);
  const tag = await prisma.tag.create({data: req.body});
  responde(res, 201, "tag succesfully added", tag);
}

export async function update_one(req: Request, res: Response) {
  assert(req.body, TagUpdateData);

  const tag = await prisma.tag.update({
    where: { id: parseInt(req.params.tag_id) },
    data: req.body,
  });
  responde(res, 200, "tag succesfully updated.", tag);
}

export async function delete_one(req: Request, res: Response) {
  await prisma.tag.delete({ where: { id: parseInt(req.params.tag_id) } });
  responde(res, 200, "tag succesfully deleted.");
}