import { prisma } from '../db';
import type { Request, Response } from 'express';
import { HttpError } from '../error';
import { assert } from 'superstruct';
import { ProjectUpdateData } from "../validation/project";
import { responde } from "../functions";

export async function update_one(req: Request, res: Response) {
    
    assert(req.body, ProjectUpdateData);
    
    const project = await prisma.project.update({
      where: { id: Number(req.params.project_id) },
      data: req.body,
      include: {
        tasks: true,
      }
    });
    if (project == null) {
      throw new HttpError("Project doesn't exist", 404);
    }
    responde(res, 200, "Project succesfully updated.", project);
}


export async function delete_one(req: Request, res: Response) {
    const project = await prisma.project.delete({
        where: { id: Number(req.params.project_id) }
    });
    if (project == null) {
      throw new HttpError("Project doesn't exist", 404);
    }
    responde(res, 204, "Project succesfully deleted.");
}

export async function get_project_tasks(req: Request, res: Response) {
  const project = await prisma.project.findUnique({
    where: { id: Number(req.params.project_id) },
    include: {
      tasks: true,
    },
  });
  if (project == null) {
    throw new HttpError("Project doesn't exist", 404);
  }
  res.status(200).json(project.tasks);
}

export async function create_project_task(req: Request, res: Response) {
  const project = await prisma.project.update({
    where: { id: Number(req.params.project_id) },
    data: {
      tasks: {
        create: req.body,
      },
    },
    include: {
      tasks: true,
    },
  });

  if (project == null) {
    throw new HttpError("Project doesn't exist", 404);
  }
  responde(res, 201, "Task succesfully created.", project.tasks);
}
