import { prisma } from '../db';
import { Prisma } from '@prisma/client';
import type { Request, Response } from 'express';
import { HttpError } from '../error';
import { assert } from 'superstruct';
import { TaskUpdateData } from "../validation/task";
import { responde } from "../functions";

export async function update_one(req: Request, res: Response) {
    assert(req.body, TaskUpdateData);
    const task = await prisma.task.update({
        where: { id: Number(req.params.task_id) },
        data: req.body,
    });
    if (task == null) {
        throw new HttpError("Task doesn't exist", 404);
    }
    res.status(200).json(task);
}

export async function delete_one(req: Request, res: Response) {
    const task = await prisma.task.delete({
        where: { id: Number(req.params.task_id) }
    });
    if (task == null) {
        throw new HttpError("Task doesn't exist", 404);
    }
    responde(res, 204, "Task succesfully deleted.");
}

export async function get_assignees(req: Request, res: Response) {
    const task = await prisma.task.findUnique({
        where: { id: Number(req.params.task_id) },
        include: {
            assignees: true,
        },
    });
    if (task == null) {
        throw new HttpError("Task doesn't exist", 404);
    }
    res.status(200).json(task.assignees);
}